package java.awt.image;

import java.awt.geom.Point2D;
import java.awt.Rectangle;
import java.awt.geom.Rectangle2D;
import java.awt.Graphics2D;
import java.awt.AlphaComposite;
import sun.awt.image.ImagingLib;
import java.awt.RenderingHints;
import java.awt.geom.AffineTransform;

public class AffineTransformOp implements BufferedImageOp, RasterOp
{
    private AffineTransform xform;
    RenderingHints hints;
    public static final int TYPE_NEAREST_NEIGHBOR = 1;
    public static final int TYPE_BILINEAR = 2;
    public static final int TYPE_BICUBIC = 3;
    int interpolationType;
    
    public AffineTransformOp(AffineTransform affineTransform, RenderingHints hints) {
        this.interpolationType = 1;
        this.validateTransform(affineTransform);
        this.xform = (AffineTransform)affineTransform.clone();
        this.hints = hints;
        if (hints != null) {
            Object value = hints.get(RenderingHints.KEY_INTERPOLATION);
            if (value == null) {
                Object value2 = hints.get(RenderingHints.KEY_RENDERING);
                if (value2 == RenderingHints.VALUE_RENDER_SPEED) {
                    this.interpolationType = 1;
                }
                else if (value2 == RenderingHints.VALUE_RENDER_QUALITY) {
                    this.interpolationType = 2;
                }
            }
            else if (value == RenderingHints.VALUE_INTERPOLATION_NEAREST_NEIGHBOR) {
                this.interpolationType = 1;
            }
            else if (value == RenderingHints.VALUE_INTERPOLATION_BILINEAR) {
                this.interpolationType = 2;
            }
            else if (value == RenderingHints.VALUE_INTERPOLATION_BICUBIC) {
                this.interpolationType = 3;
            }
        }
        else {
            this.interpolationType = 1;
        }
    }
    
    public AffineTransformOp(AffineTransform affineTransform, int n) {
        this.interpolationType = 1;
        this.validateTransform(affineTransform);
        this.xform = (AffineTransform)affineTransform.clone();
        switch (n) {
            case 1:
            case 2:
            case 3: {
                this.interpolationType = n;
                return;
            }
            default: {
                throw new IllegalArgumentException("Unknown interpolation type: " + n);
            }
        }
    }
    
    public final int getInterpolationType() {
        return this.interpolationType;
    }
    
    @Override
    public final BufferedImage filter(BufferedImage filter, BufferedImage dst) {
        if (filter == null) {
            throw new NullPointerException("src image is null");
        }
        if (filter == dst) {
            throw new IllegalArgumentException("src image cannot be the same as the dst image");
        }
        boolean n = false;
        ColorModel colorModel = filter.getColorModel();
        if ((dst = dst) == null) {
            dst = this.createCompatibleDestImage(filter, null);
        }
        else {
            ColorModel colorModel2 = dst.getColorModel();
            if (colorModel.getColorSpace().getType() != colorModel2.getColorSpace().getType()) {
                int type;
                int n2 = type = this.xform.getType();
                AffineTransform xform = this.xform;
                int n3 = 24;
                AffineTransform xform2 = this.xform;
                int n4 = (type & (n3 | 0x20)) != 0x0;
                if (n4 == 0) {
                    int n5 = n2;
                    AffineTransform xform3 = this.xform;
                    if (n5 != 1) {
                        int n6 = n2;
                        AffineTransform xform4 = this.xform;
                        if (n6 != 0) {
                            double[] flatmatrix = new double[4];
                            this.xform.getMatrix(flatmatrix);
                            n4 = (flatmatrix[0] != (double)(int)flatmatrix[0] || flatmatrix[3] != (double)(int)flatmatrix[3]);
                        }
                    }
                }
                if (n4 && colorModel.getTransparency() == 1) {
                    ColorConvertOp o = new ColorConvertOp(this.hints);
                    int width = filter.getWidth();
                    int height = filter.getHeight();
                    Object o2;
                    if (colorModel2.getTransparency() == 1) {
                        o2 = new BufferedImage(width, height, 2);
                    }
                    else {
                        o2 = new BufferedImage(colorModel2, colorModel2.createCompatibleWritableRaster(width, height), colorModel2.isAlphaPremultiplied(), null);
                    }
                    filter = o.filter(filter, o2);
                }
                else {
                    n = true;
                    dst = this.createCompatibleDestImage(filter, null);
                }
            }
        }
        if (this.interpolationType != 1 && dst.getColorModel() instanceof IndexColorModel) {
            dst = new BufferedImage(dst.getWidth(), dst.getHeight(), 2);
        }
        if (ImagingLib.filter(this, filter, dst) == null) {
            throw new ImagingOpException("Unable to transform src image");
        }
        if (!(n == 0)) {
            new ColorConvertOp(this.hints).filter(dst, dst);
        }
        else if (dst != dst) {
            Graphics2D graphics = dst.createGraphics();
            try {
                graphics.setComposite(AlphaComposite.Src);
                graphics.drawImage(dst, 0, 0, null);
            }
            finally {
                graphics.dispose();
            }
        }
        return dst;
    }
    
    @Override
    public final WritableRaster filter(Raster src, WritableRaster compatibleDestRaster) {
        if (src == null) {
            throw new NullPointerException("src image is null");
        }
        if (compatibleDestRaster == null) {
            compatibleDestRaster = this.createCompatibleDestRaster(src);
        }
        if (src == compatibleDestRaster) {
            throw new IllegalArgumentException("src image cannot be the same as the dst image");
        }
        if (src.getNumBands() != compatibleDestRaster.getNumBands()) {
            throw new IllegalArgumentException("Number of src bands (" + src.getNumBands() + ") does not match number of  dst bands (" + compatibleDestRaster.getNumBands() + ")");
        }
        if (ImagingLib.filter(this, src, compatibleDestRaster) == null) {
            throw new ImagingOpException("Unable to transform src image");
        }
        return compatibleDestRaster;
    }
    
    @Override
    public final Rectangle2D getBounds2D(BufferedImage bufferedImage) {
        return this.getBounds2D(bufferedImage.getRaster());
    }
    
    @Override
    public final Rectangle2D getBounds2D(Raster raster) {
        int width = raster.getWidth();
        int height = raster.getHeight();
        float[] array = { 0.0f, 0.0f, (float)width, 0.0f, (float)width, (float)height, 0.0f, (float)height };
        this.xform.transform(array, 0, array, 0, 4);
        float n = array[0];
        float n2 = array[1];
        float x = array[0];
        float y = array[1];
        for (int i = 2; i < 8; i += 2) {
            if (array[i] > n) {
                n = array[i];
            }
            else if (array[i] < x) {
                x = array[i];
            }
            if (array[i + 1] > n2) {
                n2 = array[i + 1];
            }
            else if (array[i + 1] < y) {
                y = array[i + 1];
            }
        }
        return new Rectangle2D.Float(x, y, n - x, n2 - y);
    }
    
    @Override
    public BufferedImage createCompatibleDestImage(BufferedImage bufferedImage, ColorModel colorModel) {
        Rectangle bounds = this.getBounds2D(bufferedImage).getBounds();
        int i = bounds.x + bounds.width;
        int j = bounds.y + bounds.height;
        if (i <= 0) {
            throw new RasterFormatException("Transformed width (" + i + ") is less than or equal to 0.");
        }
        if (j <= 0) {
            throw new RasterFormatException("Transformed height (" + j + ") is less than or equal to 0.");
        }
        if (colorModel == null) {
            ColorModel colorModel2 = bufferedImage.getColorModel();
            BufferedImage bufferedImage2;
            if (this.interpolationType != 1 && (colorModel2 instanceof IndexColorModel || colorModel2.getTransparency() == 1)) {
                bufferedImage2 = new BufferedImage(i, j, 2);
            }
            else {
                bufferedImage2 = new BufferedImage(colorModel2, bufferedImage.getRaster().createCompatibleWritableRaster(i, j), colorModel2.isAlphaPremultiplied(), null);
            }
            return bufferedImage2;
        }
        else {
            return new BufferedImage(colorModel, colorModel.createCompatibleWritableRaster(i, j), colorModel.isAlphaPremultiplied(), null);
        }
    }
    
    @Override
    public WritableRaster createCompatibleDestRaster(Raster raster) {
        Rectangle2D bounds2D = this.getBounds2D(raster);
        return raster.createCompatibleWritableRaster((int)bounds2D.getX(), (int)bounds2D.getY(), (int)bounds2D.getWidth(), (int)bounds2D.getHeight());
    }
    
    @Override
    public final Point2D getPoint2D(Point2D ptSrc, Point2D ptDst) {
        return this.xform.transform(ptSrc, ptDst);
    }
    
    public final AffineTransform getTransform() {
        return (AffineTransform)this.xform.clone();
    }
    
    @Override
    public final RenderingHints getRenderingHints() {
        if (this.hints == null) {
            Object value;
            switch (this.interpolationType) {
                case 1: {
                    value = RenderingHints.VALUE_INTERPOLATION_NEAREST_NEIGHBOR;
                    break;
                }
                case 2: {
                    value = RenderingHints.VALUE_INTERPOLATION_BILINEAR;
                    break;
                }
                case 3: {
                    value = RenderingHints.VALUE_INTERPOLATION_BICUBIC;
                    break;
                }
                default: {
                    throw new InternalError("Unknown interpolation type " + this.interpolationType);
                }
            }
            this.hints = new RenderingHints(RenderingHints.KEY_INTERPOLATION, value);
        }
        return this.hints;
    }
    
    void validateTransform(AffineTransform obj) {
        if (Math.abs(obj.getDeterminant()) <= Double.MIN_VALUE) {
            throw new ImagingOpException("Unable to invert transform " + obj);
        }
        return;
    }
}
